# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import __libarchive
else:
    import __libarchive

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def archive_read_new():
    return __libarchive.archive_read_new()

def archive_read_free(arg1):
    return __libarchive.archive_read_free(arg1)

def archive_read_open_filename(arg1, _filename, _block_size):
    return __libarchive.archive_read_open_filename(arg1, _filename, _block_size)

def archive_read_open_memory(arg1, buff, size):
    return __libarchive.archive_read_open_memory(arg1, buff, size)

def archive_read_open_memory2(a, buff, size, read_size):
    return __libarchive.archive_read_open_memory2(a, buff, size, read_size)

def archive_read_open_fd(arg1, _fd, _block_size):
    return __libarchive.archive_read_open_fd(arg1, _fd, _block_size)

def archive_read_close(arg1):
    return __libarchive.archive_read_close(arg1)

def archive_format(arg1):
    return __libarchive.archive_format(arg1)

def archive_read_next_header2(arg1, arg2):
    return __libarchive.archive_read_next_header2(arg1, arg2)

def archive_entry_stat(arg1):
    return __libarchive.archive_entry_stat(arg1)

def archive_read_header_position(arg1):
    return __libarchive.archive_read_header_position(arg1)

def archive_read_set_format_option(_a, m, o, v):
    return __libarchive.archive_read_set_format_option(_a, m, o, v)

def archive_read_set_filter_option(_a, m, o, v):
    return __libarchive.archive_read_set_filter_option(_a, m, o, v)

def archive_read_set_option(_a, m, o, v):
    return __libarchive.archive_read_set_option(_a, m, o, v)

def archive_read_set_options(_a, opts):
    return __libarchive.archive_read_set_options(_a, opts)

def archive_read_add_passphrase(arg1, arg2):
    return __libarchive.archive_read_add_passphrase(arg1, arg2)

def archive_read_data_skip(arg1):
    return __libarchive.archive_read_data_skip(arg1)

def archive_read_data_into_fd(arg1, fd):
    return __libarchive.archive_read_data_into_fd(arg1, fd)

def archive_read_support_compression_all(arg1):
    return __libarchive.archive_read_support_compression_all(arg1)

def archive_read_support_compression_bzip2(arg1):
    return __libarchive.archive_read_support_compression_bzip2(arg1)

def archive_read_support_compression_compress(arg1):
    return __libarchive.archive_read_support_compression_compress(arg1)

def archive_read_support_compression_gzip(arg1):
    return __libarchive.archive_read_support_compression_gzip(arg1)

def archive_read_support_compression_lzip(arg1):
    return __libarchive.archive_read_support_compression_lzip(arg1)

def archive_read_support_compression_lzma(arg1):
    return __libarchive.archive_read_support_compression_lzma(arg1)

def archive_read_support_compression_none(arg1):
    return __libarchive.archive_read_support_compression_none(arg1)

def archive_read_support_compression_program(arg1, command):
    return __libarchive.archive_read_support_compression_program(arg1, command)

def archive_read_support_compression_program_signature(arg1, arg2, arg3, arg4):
    return __libarchive.archive_read_support_compression_program_signature(arg1, arg2, arg3, arg4)

def archive_read_support_compression_rpm(arg1):
    return __libarchive.archive_read_support_compression_rpm(arg1)

def archive_read_support_compression_uu(arg1):
    return __libarchive.archive_read_support_compression_uu(arg1)

def archive_read_support_compression_xz(arg1):
    return __libarchive.archive_read_support_compression_xz(arg1)

def archive_read_support_filter_all(arg1):
    return __libarchive.archive_read_support_filter_all(arg1)

def archive_read_support_filter_bzip2(arg1):
    return __libarchive.archive_read_support_filter_bzip2(arg1)

def archive_read_support_filter_compress(arg1):
    return __libarchive.archive_read_support_filter_compress(arg1)

def archive_read_support_filter_gzip(arg1):
    return __libarchive.archive_read_support_filter_gzip(arg1)

def archive_read_support_filter_grzip(arg1):
    return __libarchive.archive_read_support_filter_grzip(arg1)

def archive_read_support_filter_lrzip(arg1):
    return __libarchive.archive_read_support_filter_lrzip(arg1)

def archive_read_support_filter_lz4(arg1):
    return __libarchive.archive_read_support_filter_lz4(arg1)

def archive_read_support_filter_lzip(arg1):
    return __libarchive.archive_read_support_filter_lzip(arg1)

def archive_read_support_filter_lzma(arg1):
    return __libarchive.archive_read_support_filter_lzma(arg1)

def archive_read_support_filter_lzop(arg1):
    return __libarchive.archive_read_support_filter_lzop(arg1)

def archive_read_support_filter_none(arg1):
    return __libarchive.archive_read_support_filter_none(arg1)

def archive_read_support_filter_program(arg1, command):
    return __libarchive.archive_read_support_filter_program(arg1, command)

def archive_read_support_filter_program_signature(arg1, arg2, arg3, arg4):
    return __libarchive.archive_read_support_filter_program_signature(arg1, arg2, arg3, arg4)

def archive_read_support_filter_rpm(arg1):
    return __libarchive.archive_read_support_filter_rpm(arg1)

def archive_read_support_filter_uu(arg1):
    return __libarchive.archive_read_support_filter_uu(arg1)

def archive_read_support_filter_xz(arg1):
    return __libarchive.archive_read_support_filter_xz(arg1)

def archive_read_support_format_7zip(arg1):
    return __libarchive.archive_read_support_format_7zip(arg1)

def archive_read_support_format_all(arg1):
    return __libarchive.archive_read_support_format_all(arg1)

def archive_read_support_format_ar(arg1):
    return __libarchive.archive_read_support_format_ar(arg1)

def archive_read_support_format_by_code(arg1, arg2):
    return __libarchive.archive_read_support_format_by_code(arg1, arg2)

def archive_read_support_format_cab(arg1):
    return __libarchive.archive_read_support_format_cab(arg1)

def archive_read_support_format_cpio(arg1):
    return __libarchive.archive_read_support_format_cpio(arg1)

def archive_read_support_format_empty(arg1):
    return __libarchive.archive_read_support_format_empty(arg1)

def archive_read_support_format_gnutar(arg1):
    return __libarchive.archive_read_support_format_gnutar(arg1)

def archive_read_support_format_iso9660(arg1):
    return __libarchive.archive_read_support_format_iso9660(arg1)

def archive_read_support_format_lha(arg1):
    return __libarchive.archive_read_support_format_lha(arg1)

def archive_read_support_format_rar(arg1):
    return __libarchive.archive_read_support_format_rar(arg1)

def archive_read_support_format_raw(arg1):
    return __libarchive.archive_read_support_format_raw(arg1)

def archive_read_support_format_tar(arg1):
    return __libarchive.archive_read_support_format_tar(arg1)

def archive_read_support_format_warc(arg1):
    return __libarchive.archive_read_support_format_warc(arg1)

def archive_read_support_format_xar(arg1):
    return __libarchive.archive_read_support_format_xar(arg1)

def archive_read_support_format_zip(arg1):
    return __libarchive.archive_read_support_format_zip(arg1)

def archive_read_support_format_zip_streamable(arg1):
    return __libarchive.archive_read_support_format_zip_streamable(arg1)

def archive_read_support_format_zip_seekable(arg1):
    return __libarchive.archive_read_support_format_zip_seekable(arg1)

def archive_read_set_format(arg1, arg2):
    return __libarchive.archive_read_set_format(arg1, arg2)

def archive_read_append_filter(arg1, arg2):
    return __libarchive.archive_read_append_filter(arg1, arg2)

def archive_read_append_filter_program(arg1, arg2):
    return __libarchive.archive_read_append_filter_program(arg1, arg2)

def archive_read_append_filter_program_signature(arg1, arg2, arg3, arg4):
    return __libarchive.archive_read_append_filter_program_signature(arg1, arg2, arg3, arg4)

def archive_write_new():
    return __libarchive.archive_write_new()

def archive_write_free(arg1):
    return __libarchive.archive_write_free(arg1)

def archive_write_open(arg1, arg2, arg3, arg4, arg5):
    return __libarchive.archive_write_open(arg1, arg2, arg3, arg4, arg5)

def archive_write_open_fd(arg1, _fd):
    return __libarchive.archive_write_open_fd(arg1, _fd)

def archive_write_open_filename(arg1, _file):
    return __libarchive.archive_write_open_filename(arg1, _file)

def archive_write_open_filename_w(arg1, _file):
    return __libarchive.archive_write_open_filename_w(arg1, _file)

def archive_write_open_memory(arg1, _buffer, _buffSize, _used):
    return __libarchive.archive_write_open_memory(arg1, _buffer, _buffSize, _used)

def archive_write_close(arg1):
    return __libarchive.archive_write_close(arg1)

def archive_write_header(arg1, arg2):
    return __libarchive.archive_write_header(arg1, arg2)

def archive_write_set_format_option(_a, m, o, v):
    return __libarchive.archive_write_set_format_option(_a, m, o, v)

def archive_write_set_filter_option(_a, m, o, v):
    return __libarchive.archive_write_set_filter_option(_a, m, o, v)

def archive_write_set_option(_a, m, o, v):
    return __libarchive.archive_write_set_option(_a, m, o, v)

def archive_write_set_options(_a, opts):
    return __libarchive.archive_write_set_options(_a, opts)

def archive_write_set_passphrase(_a, p):
    return __libarchive.archive_write_set_passphrase(_a, p)

def archive_write_finish_entry(arg1):
    return __libarchive.archive_write_finish_entry(arg1)

def archive_write_add_filter(arg1, filter_code):
    return __libarchive.archive_write_add_filter(arg1, filter_code)

def archive_write_add_filter_by_name(arg1, name):
    return __libarchive.archive_write_add_filter_by_name(arg1, name)

def archive_write_add_filter_b64encode(arg1):
    return __libarchive.archive_write_add_filter_b64encode(arg1)

def archive_write_add_filter_bzip2(arg1):
    return __libarchive.archive_write_add_filter_bzip2(arg1)

def archive_write_add_filter_compress(arg1):
    return __libarchive.archive_write_add_filter_compress(arg1)

def archive_write_add_filter_grzip(arg1):
    return __libarchive.archive_write_add_filter_grzip(arg1)

def archive_write_add_filter_gzip(arg1):
    return __libarchive.archive_write_add_filter_gzip(arg1)

def archive_write_add_filter_lrzip(arg1):
    return __libarchive.archive_write_add_filter_lrzip(arg1)

def archive_write_add_filter_lz4(arg1):
    return __libarchive.archive_write_add_filter_lz4(arg1)

def archive_write_add_filter_lzip(arg1):
    return __libarchive.archive_write_add_filter_lzip(arg1)

def archive_write_add_filter_lzma(arg1):
    return __libarchive.archive_write_add_filter_lzma(arg1)

def archive_write_add_filter_lzop(arg1):
    return __libarchive.archive_write_add_filter_lzop(arg1)

def archive_write_add_filter_none(arg1):
    return __libarchive.archive_write_add_filter_none(arg1)

def archive_write_add_filter_program(arg1, cmd):
    return __libarchive.archive_write_add_filter_program(arg1, cmd)

def archive_write_add_filter_uuencode(arg1):
    return __libarchive.archive_write_add_filter_uuencode(arg1)

def archive_write_add_filter_xz(arg1):
    return __libarchive.archive_write_add_filter_xz(arg1)

def archive_write_set_format(arg1, format_code):
    return __libarchive.archive_write_set_format(arg1, format_code)

def archive_write_set_format_by_name(arg1, name):
    return __libarchive.archive_write_set_format_by_name(arg1, name)

def archive_write_set_format_7zip(arg1):
    return __libarchive.archive_write_set_format_7zip(arg1)

def archive_write_set_format_ar_bsd(arg1):
    return __libarchive.archive_write_set_format_ar_bsd(arg1)

def archive_write_set_format_ar_svr4(arg1):
    return __libarchive.archive_write_set_format_ar_svr4(arg1)

def archive_write_set_format_cpio(arg1):
    return __libarchive.archive_write_set_format_cpio(arg1)

def archive_write_set_format_cpio_newc(arg1):
    return __libarchive.archive_write_set_format_cpio_newc(arg1)

def archive_write_set_format_gnutar(arg1):
    return __libarchive.archive_write_set_format_gnutar(arg1)

def archive_write_set_format_iso9660(arg1):
    return __libarchive.archive_write_set_format_iso9660(arg1)

def archive_write_set_format_mtree(arg1):
    return __libarchive.archive_write_set_format_mtree(arg1)

def archive_write_set_format_mtree_classic(arg1):
    return __libarchive.archive_write_set_format_mtree_classic(arg1)

def archive_write_set_format_pax(arg1):
    return __libarchive.archive_write_set_format_pax(arg1)

def archive_write_set_format_pax_restricted(arg1):
    return __libarchive.archive_write_set_format_pax_restricted(arg1)

def archive_write_set_format_raw(arg1):
    return __libarchive.archive_write_set_format_raw(arg1)

def archive_write_set_format_shar(arg1):
    return __libarchive.archive_write_set_format_shar(arg1)

def archive_write_set_format_shar_dump(arg1):
    return __libarchive.archive_write_set_format_shar_dump(arg1)

def archive_write_set_format_ustar(arg1):
    return __libarchive.archive_write_set_format_ustar(arg1)

def archive_write_set_format_v7tar(arg1):
    return __libarchive.archive_write_set_format_v7tar(arg1)

def archive_write_set_format_warc(arg1):
    return __libarchive.archive_write_set_format_warc(arg1)

def archive_write_set_format_xar(arg1):
    return __libarchive.archive_write_set_format_xar(arg1)

def archive_write_set_format_zip(arg1):
    return __libarchive.archive_write_set_format_zip(arg1)

def archive_write_set_format_filter_by_ext(a, filename):
    return __libarchive.archive_write_set_format_filter_by_ext(a, filename)

def archive_write_set_format_filter_by_ext_def(a, filename, def_ext):
    return __libarchive.archive_write_set_format_filter_by_ext_def(a, filename, def_ext)

def archive_write_zip_set_compression_deflate(arg1):
    return __libarchive.archive_write_zip_set_compression_deflate(arg1)

def archive_write_zip_set_compression_store(arg1):
    return __libarchive.archive_write_zip_set_compression_store(arg1)

def archive_entry_new():
    return __libarchive.archive_entry_new()

def archive_entry_free(arg1):
    return __libarchive.archive_entry_free(arg1)

def archive_entry_pathname(arg1):
    return __libarchive.archive_entry_pathname(arg1)

def archive_entry_pathname_w(arg1):
    return __libarchive.archive_entry_pathname_w(arg1)

def archive_entry_size(arg1):
    return __libarchive.archive_entry_size(arg1)

def archive_entry_mtime(arg1):
    return __libarchive.archive_entry_mtime(arg1)

def archive_entry_filetype(arg1):
    return __libarchive.archive_entry_filetype(arg1)

def archive_entry_perm(arg1):
    return __libarchive.archive_entry_perm(arg1)

def archive_entry_symlink(arg1):
    return __libarchive.archive_entry_symlink(arg1)

def archive_entry_set_link(arg1, arg2):
    return __libarchive.archive_entry_set_link(arg1, arg2)

def archive_entry_symlink_w(arg1):
    return __libarchive.archive_entry_symlink_w(arg1)

def archive_read_disk_set_symlink_logical(arg1):
    return __libarchive.archive_read_disk_set_symlink_logical(arg1)

def archive_read_disk_set_symlink_physical(arg1):
    return __libarchive.archive_read_disk_set_symlink_physical(arg1)

def archive_read_disk_set_symlink_hybrid(arg1):
    return __libarchive.archive_read_disk_set_symlink_hybrid(arg1)

def archive_entry_set_symlink(arg1, arg2):
    return __libarchive.archive_entry_set_symlink(arg1, arg2)

def archive_entry_copy_symlink(arg1, arg2):
    return __libarchive.archive_entry_copy_symlink(arg1, arg2)

def archive_entry_copy_symlink_w(arg1, arg2):
    return __libarchive.archive_entry_copy_symlink_w(arg1, arg2)

def archive_entry_set_pathname(arg1, arg2):
    return __libarchive.archive_entry_set_pathname(arg1, arg2)

def archive_entry_set_size(arg1, arg2):
    return __libarchive.archive_entry_set_size(arg1, arg2)

def archive_entry_set_mtime(arg1, arg2, arg3):
    return __libarchive.archive_entry_set_mtime(arg1, arg2, arg3)

def archive_entry_set_filetype(arg1, arg2):
    return __libarchive.archive_entry_set_filetype(arg1, arg2)

def archive_entry_set_perm(arg1, arg2):
    return __libarchive.archive_entry_set_perm(arg1, arg2)

def archive_errno(arg1):
    return __libarchive.archive_errno(arg1)

def archive_error_string(arg1):
    return __libarchive.archive_error_string(arg1)

def archive_version_number():
    return __libarchive.archive_version_number()

def archive_version_string():
    return __libarchive.archive_version_string()
ARCHIVE_EOF = __libarchive.ARCHIVE_EOF
ARCHIVE_OK = __libarchive.ARCHIVE_OK
ARCHIVE_RETRY = __libarchive.ARCHIVE_RETRY
ARCHIVE_WARN = __libarchive.ARCHIVE_WARN
ARCHIVE_FAILED = __libarchive.ARCHIVE_FAILED
ARCHIVE_FATAL = __libarchive.ARCHIVE_FATAL
ARCHIVE_FILTER_NONE = __libarchive.ARCHIVE_FILTER_NONE
ARCHIVE_FILTER_GZIP = __libarchive.ARCHIVE_FILTER_GZIP
ARCHIVE_FILTER_BZIP2 = __libarchive.ARCHIVE_FILTER_BZIP2
ARCHIVE_FILTER_COMPRESS = __libarchive.ARCHIVE_FILTER_COMPRESS
ARCHIVE_FILTER_PROGRAM = __libarchive.ARCHIVE_FILTER_PROGRAM
ARCHIVE_FILTER_LZMA = __libarchive.ARCHIVE_FILTER_LZMA
ARCHIVE_FILTER_XZ = __libarchive.ARCHIVE_FILTER_XZ
ARCHIVE_FILTER_UU = __libarchive.ARCHIVE_FILTER_UU
ARCHIVE_FILTER_RPM = __libarchive.ARCHIVE_FILTER_RPM
ARCHIVE_FILTER_LZIP = __libarchive.ARCHIVE_FILTER_LZIP
ARCHIVE_FILTER_LRZIP = __libarchive.ARCHIVE_FILTER_LRZIP
ARCHIVE_FILTER_LZOP = __libarchive.ARCHIVE_FILTER_LZOP
ARCHIVE_FILTER_GRZIP = __libarchive.ARCHIVE_FILTER_GRZIP
ARCHIVE_FILTER_LZ4 = __libarchive.ARCHIVE_FILTER_LZ4
ARCHIVE_FORMAT_BASE_MASK = __libarchive.ARCHIVE_FORMAT_BASE_MASK
ARCHIVE_FORMAT_CPIO = __libarchive.ARCHIVE_FORMAT_CPIO
ARCHIVE_FORMAT_CPIO_POSIX = __libarchive.ARCHIVE_FORMAT_CPIO_POSIX
ARCHIVE_FORMAT_CPIO_BIN_LE = __libarchive.ARCHIVE_FORMAT_CPIO_BIN_LE
ARCHIVE_FORMAT_CPIO_BIN_BE = __libarchive.ARCHIVE_FORMAT_CPIO_BIN_BE
ARCHIVE_FORMAT_CPIO_SVR4_NOCRC = __libarchive.ARCHIVE_FORMAT_CPIO_SVR4_NOCRC
ARCHIVE_FORMAT_CPIO_SVR4_CRC = __libarchive.ARCHIVE_FORMAT_CPIO_SVR4_CRC
ARCHIVE_FORMAT_CPIO_AFIO_LARGE = __libarchive.ARCHIVE_FORMAT_CPIO_AFIO_LARGE
ARCHIVE_FORMAT_SHAR = __libarchive.ARCHIVE_FORMAT_SHAR
ARCHIVE_FORMAT_SHAR_BASE = __libarchive.ARCHIVE_FORMAT_SHAR_BASE
ARCHIVE_FORMAT_SHAR_DUMP = __libarchive.ARCHIVE_FORMAT_SHAR_DUMP
ARCHIVE_FORMAT_TAR = __libarchive.ARCHIVE_FORMAT_TAR
ARCHIVE_FORMAT_TAR_USTAR = __libarchive.ARCHIVE_FORMAT_TAR_USTAR
ARCHIVE_FORMAT_TAR_PAX_INTERCHANGE = __libarchive.ARCHIVE_FORMAT_TAR_PAX_INTERCHANGE
ARCHIVE_FORMAT_TAR_PAX_RESTRICTED = __libarchive.ARCHIVE_FORMAT_TAR_PAX_RESTRICTED
ARCHIVE_FORMAT_TAR_GNUTAR = __libarchive.ARCHIVE_FORMAT_TAR_GNUTAR
ARCHIVE_FORMAT_ISO9660 = __libarchive.ARCHIVE_FORMAT_ISO9660
ARCHIVE_FORMAT_ISO9660_ROCKRIDGE = __libarchive.ARCHIVE_FORMAT_ISO9660_ROCKRIDGE
ARCHIVE_FORMAT_ZIP = __libarchive.ARCHIVE_FORMAT_ZIP
ARCHIVE_FORMAT_EMPTY = __libarchive.ARCHIVE_FORMAT_EMPTY
ARCHIVE_FORMAT_AR = __libarchive.ARCHIVE_FORMAT_AR
ARCHIVE_FORMAT_AR_GNU = __libarchive.ARCHIVE_FORMAT_AR_GNU
ARCHIVE_FORMAT_AR_BSD = __libarchive.ARCHIVE_FORMAT_AR_BSD
ARCHIVE_FORMAT_MTREE = __libarchive.ARCHIVE_FORMAT_MTREE
ARCHIVE_FORMAT_RAW = __libarchive.ARCHIVE_FORMAT_RAW
ARCHIVE_FORMAT_XAR = __libarchive.ARCHIVE_FORMAT_XAR
ARCHIVE_FORMAT_LHA = __libarchive.ARCHIVE_FORMAT_LHA
ARCHIVE_FORMAT_CAB = __libarchive.ARCHIVE_FORMAT_CAB
ARCHIVE_FORMAT_RAR = __libarchive.ARCHIVE_FORMAT_RAR
ARCHIVE_FORMAT_7ZIP = __libarchive.ARCHIVE_FORMAT_7ZIP
ARCHIVE_FORMAT_WARC = __libarchive.ARCHIVE_FORMAT_WARC
ARCHIVE_EXTRACT_OWNER = __libarchive.ARCHIVE_EXTRACT_OWNER
ARCHIVE_EXTRACT_PERM = __libarchive.ARCHIVE_EXTRACT_PERM
ARCHIVE_EXTRACT_TIME = __libarchive.ARCHIVE_EXTRACT_TIME
ARCHIVE_EXTRACT_NO_OVERWRITE = __libarchive.ARCHIVE_EXTRACT_NO_OVERWRITE
ARCHIVE_EXTRACT_UNLINK = __libarchive.ARCHIVE_EXTRACT_UNLINK
ARCHIVE_EXTRACT_ACL = __libarchive.ARCHIVE_EXTRACT_ACL
ARCHIVE_EXTRACT_FFLAGS = __libarchive.ARCHIVE_EXTRACT_FFLAGS
ARCHIVE_EXTRACT_XATTR = __libarchive.ARCHIVE_EXTRACT_XATTR
ARCHIVE_EXTRACT_SECURE_SYMLINKS = __libarchive.ARCHIVE_EXTRACT_SECURE_SYMLINKS
ARCHIVE_EXTRACT_SECURE_NODOTDOT = __libarchive.ARCHIVE_EXTRACT_SECURE_NODOTDOT
ARCHIVE_EXTRACT_NO_AUTODIR = __libarchive.ARCHIVE_EXTRACT_NO_AUTODIR
ARCHIVE_EXTRACT_NO_OVERWRITE_NEWER = __libarchive.ARCHIVE_EXTRACT_NO_OVERWRITE_NEWER
ARCHIVE_EXTRACT_SPARSE = __libarchive.ARCHIVE_EXTRACT_SPARSE
ARCHIVE_EXTRACT_MAC_METADATA = __libarchive.ARCHIVE_EXTRACT_MAC_METADATA
ARCHIVE_EXTRACT_NO_HFS_COMPRESSION = __libarchive.ARCHIVE_EXTRACT_NO_HFS_COMPRESSION
ARCHIVE_EXTRACT_HFS_COMPRESSION_FORCED = __libarchive.ARCHIVE_EXTRACT_HFS_COMPRESSION_FORCED
ARCHIVE_EXTRACT_SECURE_NOABSOLUTEPATHS = __libarchive.ARCHIVE_EXTRACT_SECURE_NOABSOLUTEPATHS
ARCHIVE_EXTRACT_CLEAR_NOCHANGE_FFLAGS = __libarchive.ARCHIVE_EXTRACT_CLEAR_NOCHANGE_FFLAGS

def archive_read_data_into_str(archive, len):
    return __libarchive.archive_read_data_into_str(archive, len)

def archive_write_data_from_str(archive, str):
    return __libarchive.archive_write_data_from_str(archive, str)





import os
from libarchive import is_archive, Entry, SeekableArchive
from tarfile import DEFAULT_FORMAT, USTAR_FORMAT, GNU_FORMAT, PAX_FORMAT, ENCODING
from tarfile import REGTYPE, SYMTYPE, DIRTYPE, FIFOTYPE, CHRTYPE, BLKTYPE

FORMAT_CONVERSION = {
    USTAR_FORMAT: 'tar',
    GNU_FORMAT: 'gnu',
    PAX_FORMAT: 'pax',
}


def is_tarfile(filename):
    return is_archive(filename, formats=('tar', 'gnu', 'pax'))


def open(**kwargs):
    return TarFile(**kwargs)


class TarInfo(Entry):
    def __init__(self, name):
        super(TarInfo, self).__init__(pathname=name)

    fromtarfile = Entry.from_archive

    def get_name(self):
        return self.pathname

    def set_name(self, value):
        self.pathname = value

    name = property(get_name, set_name)

    @property
    def get_type(self):
        for attr, type in (
            ('isdir', DIRTYPE),
            ('isfile', REGTYPE),
            ('issym', SYMTYPE),
            ('isfifo', FIFOTYPE),
            ('ischr', CHRTYPE),
            ('isblk', BLKTYPE),
        ):
            if getattr(self, attr)():
                return type

    def _get_missing(self):
        raise NotImplemented()

    def _set_missing(self, value):
        raise NotImplemented()

    pax_headers = property(_get_missing, _set_missing)


class TarFile(SeekableArchive):
    getmember = SeekableArchive.getentry
    list = SeekableArchive.printlist
    extract = SeekableArchive.readpath
    extractfile = SeekableArchive.readstream

    def __init__(self, name=None, mode='r', fileobj=None, format=DEFAULT_FORMAT, tarinfo=TarInfo, encoding=ENCODING):
        if name:
            f = name
        elif fileobj:
            f = fileobj
        try:
            format = FORMAT_CONVERSION.get(format)
        except KeyError:
            raise Exception('Invalid tar format: %s' % format)
        super(TarFile, self).__init__(f, mode=mode, format=format, entry_class=tarinfo, encoding=encoding)

    def getmembers(self):
        return list(self)

    def getnames(self):
        return list(self.iterpaths)

    def __next__(self):
        raise NotImplementedError
        pass  # TODO: how to do this?

    def extract(self, member, path=None):
        if path is None:
            path = os.getcwd()
        if isinstance(member, str):
            f = os.path.join(path, member)
        else:
            f = os.path.join(path, member.pathname)
        return self.readpath(member, f)

    def add(self, name, arcname, recursive=True, exclude=None, filter=None):
        pass  # TODO: implement this.

    def addfile(self, tarinfo, fileobj):
        return self.writepath(fileobj, tarinfo)

    def gettarinfo(self, name=None, arcname=None, fileobj=None):
        if name:
            f = name
        elif fileobj:
            f = fileobj
        entry = self.entry_class.from_file(f)
        if arcname:
            entry.pathname = arcname
        return entry

    def _get_missing(self):
        raise NotImplemented()

    def _set_missing(self, value):
        raise NotImplemented()

    pax_headers = property(_get_missing, _set_missing)



# Copyright (c) 2011, SmartFile <btimby@smartfile.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the organization nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import stat
import sys
import time
import warnings
from libarchive import _libarchive
from io import StringIO

PY3 = sys.version_info[0] >= 3

# Suggested block size for libarchive. Libarchive may adjust it.
BLOCK_SIZE = 10240

MTIME_FORMAT = ''

# Default encoding scheme.
ENCODING = 'utf-8'

# Functions to initialize read/write for various libarchive supported formats and filters.
FORMATS = {
    None: (_libarchive.archive_read_support_format_all, None),
    'tar': (_libarchive.archive_read_support_format_tar, _libarchive.archive_write_set_format_ustar),
    'pax': (_libarchive.archive_read_support_format_tar, _libarchive.archive_write_set_format_pax),
    'gnu': (_libarchive.archive_read_support_format_gnutar, _libarchive.archive_write_set_format_gnutar),
    'zip': (_libarchive.archive_read_support_format_zip, _libarchive.archive_write_set_format_zip),
    'rar': (_libarchive.archive_read_support_format_rar, None),
    '7zip': (_libarchive.archive_read_support_format_7zip, None),
    'ar': (_libarchive.archive_read_support_format_ar, None),
    'cab': (_libarchive.archive_read_support_format_cab, None),
    'cpio': (_libarchive.archive_read_support_format_cpio, _libarchive.archive_write_set_format_cpio_newc),
    'iso': (_libarchive.archive_read_support_format_iso9660, _libarchive.archive_write_set_format_iso9660),
    'lha': (_libarchive.archive_read_support_format_lha, None),
    'xar': (_libarchive.archive_read_support_format_xar, _libarchive.archive_write_set_format_xar),
}

FILTERS = {
    None: (_libarchive.archive_read_support_filter_all, _libarchive.archive_write_add_filter_none),
    'gz': (_libarchive.archive_read_support_filter_gzip, _libarchive.archive_write_add_filter_gzip),
    'bz2': (_libarchive.archive_read_support_filter_bzip2, _libarchive.archive_write_add_filter_bzip2),
}

# Map file extensions to formats and filters. To support quick detection.
FORMAT_EXTENSIONS = {
    '.tar': 'tar',
    '.zip': 'zip',
    '.rar': 'rar',
    '.7z': '7zip',
    '.ar': 'ar',
    '.cab': 'cab',
    '.rpm': 'cpio',
    '.cpio': 'cpio',
    '.iso': 'iso',
    '.lha': 'lha',
    '.xar': 'xar',
}
FILTER_EXTENSIONS = {
    '.gz': 'gz',
    '.bz2': 'bz2',
}


class EOF(Exception):
    '''Raised by ArchiveInfo.from_archive() when unable to read the next
    archive header.'''

    pass

def version():
    '''Returns the version of the libarchive library.'''
    return _libarchive.archive_version_string().split()[1]


def get_error(archive):
    '''Retrieves the last error description for the given archive instance.'''
    return _libarchive.archive_error_string(archive)


def call_and_check(func, archive, *args):
    '''Executes a libarchive function and raises an exception when appropriate.'''
    ret = func(*args)
    if ret == _libarchive.ARCHIVE_OK:
        return
    elif ret == _libarchive.ARCHIVE_WARN:
        warnings.warn('Warning executing function: %s.' % get_error(archive), RuntimeWarning)
    elif ret == _libarchive.ARCHIVE_EOF:
        raise EOF()
    else:
        raise Exception('Problem executing function, message is: %s.' % get_error(archive))


def get_func(name, items, index):
    item = items.get(name, None)
    if item is None:
        return None
    return item[index]


def guess_format(filename):
    if isinstance(filename, int):
        filename = ext = ''
    else:
        filename, ext = os.path.splitext(filename)
    filter = FILTER_EXTENSIONS.get(ext)
    if filter:
        filename, ext = os.path.splitext(filename)
    format = FORMAT_EXTENSIONS.get(ext)
    return format, filter


def is_archive_name(filename, formats=None):
    '''Quick check to see if the given file has an extension indiciating that it is
    an archive. The format parameter can be used to limit what archive format is acceptable.
    If omitted, all supported archive formats will be checked.

    This function will return the name of the most likely archive format, None if the file is
    unlikely to be an archive.'''
    if formats is None:
        formats = list(FORMAT_EXTENSIONS.values())
    format, filter = guess_format(filename)
    if format in formats:
        return format


def is_archive(f, formats=(None,), filters=(None,)):
    '''Check to see if the given file is actually an archive. The format parameter
    can be used to specify which archive format is acceptable. If ommitted, all supported
    archive formats will be checked. It opens the file using libarchive. If no error is
    received, the file was successfully detected by the libarchive bidding process.

    This procedure is quite costly, so you should avoid calling it unless you are reasonably
    sure that the given file is an archive. In other words, you may wish to filter large
    numbers of file names using is_archive_name() before double-checking the positives with
    this function.

    This function will return True if the file can be opened as an archive using the given
    format(s)/filter(s).'''
    need_close = False
    if isinstance(f, str):
        f = open(f, 'rb')
        need_close = True
    a = _libarchive.archive_read_new()
    for format in formats:
        format = get_func(format, FORMATS, 0)
        if format is None:
            return False
        format(a)
    for filter in filters:
        filter = get_func(filter, FILTERS, 0)
        if filter is None:
            return False
        filter(a)
    try:
        try:
            call_and_check(_libarchive.archive_read_open_fd, a, a, f.fileno(), BLOCK_SIZE)
            return True
        except:
            return False
    finally:
        _libarchive.archive_read_close(a)
        _libarchive.archive_read_free(a)
        if need_close:
            f.close()


class EntryReadStream(object):
    '''A file-like object for reading an entry from the archive.'''

    def __init__(self, archive, size):
        self.archive = archive
        self.closed = False
        self.size = size
        self.bytes = 0

    def __enter__(self):
        return self

    def __exit__(self, *args):
        return

    def __iter__(self):
        if self.closed:
            return
        while True:
            data = self.read(BLOCK_SIZE)
            if not data:
                break
            yield data

    def __len__(self):
        return self.size

    def tell(self):
        return self.bytes

    def read(self, bytes=-1):
        if self.closed:
            return
        if self.bytes == self.size:
            # EOF already reached.
            return
        if bytes < 0:
            bytes = self.size - self.bytes
        elif self.bytes + bytes > self.size:
            # Limit read to remaining bytes
            bytes = self.size - self.bytes
        # Read requested bytes
        data = _libarchive.archive_read_data_into_str(self.archive._a, bytes)
        self.bytes += len(data)
        return data

    def close(self):
        if self.closed:
            return
        # Call archive.close() with _defer True to let it know we have been
        # closed and it is now safe to actually close.
        self.archive.close(_defer=True)
        self.archive = None
        self.closed = True


class EntryWriteStream(object):
    '''A file-like object for writing an entry to an archive.

    If the size is known ahead of time and provided, then the file contents
    are not buffered but flushed directly to the archive. If size is omitted,
    then the file contents are buffered and flushed in the close() method.'''

    def __init__(self, archive, pathname, size=None):
        self.archive = archive
        self.entry = Entry(pathname=pathname, mtime=time.time(), mode=stat.S_IFREG)
        if size is None:
            self.buffer = StringIO()
        else:
            self.buffer = None
            self.entry.size = size
            self.entry.to_archive(self.archive)
        self.bytes = 0
        self.closed = False

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()

    def __del__(self):
        self.close()

    def __len__(self):
        return self.bytes

    def tell(self):
        return self.bytes

    def write(self, data):
        if self.closed:
            raise Exception('Cannot write to closed stream.')
        if self.buffer:
            self.buffer.write(data)
        else:
            _libarchive.archive_write_data_from_str(self.archive._a, data.encode(ENCODING))
        self.bytes += len(data)

    def close(self):
        if self.closed:
            return
        if self.buffer:
            self.entry.size = self.buffer.tell()
            self.entry.to_archive(self.archive)
            _libarchive.archive_write_data_from_str(self.archive._a, self.buffer.getvalue().encode(ENCODING))
        _libarchive.archive_write_finish_entry(self.archive._a)

        # Call archive.close() with _defer True to let it know we have been
        # closed and it is now safe to actually close.
        self.archive.close(_defer=True)
        self.archive = None
        self.closed = True


class Entry(object):
    '''An entry within an archive. Represents the header data and it's location within the archive.'''

    def __init__(self, pathname=None, size=None, mtime=None, mode=None, hpos=None, encoding=ENCODING):

        self.pathname = pathname
        self.size = size
        self.mtime = mtime
        self.mode = mode
        self.hpos = hpos
        self.encoding = encoding

        self.symlink = ""

    @property
    def header_position(self):
        return self.hpos

    @classmethod
    def from_archive(cls, archive, encoding=ENCODING):
        '''Instantiates an Entry class and sets all the properties from an archive header.'''
        e = _libarchive.archive_entry_new()
        try:
            call_and_check(_libarchive.archive_read_next_header2, archive._a, archive._a, e)
            mode = _libarchive.archive_entry_filetype(e)
            mode |= _libarchive.archive_entry_perm(e)

            if PY3:
                pathname = _libarchive.archive_entry_pathname(e)
            else:
                pathname = _libarchive.archive_entry_pathname(e).decode(encoding)

            entry = cls(
                pathname=pathname,
                size=_libarchive.archive_entry_size(e),
                mtime=_libarchive.archive_entry_mtime(e),
                mode=mode,
                hpos=archive.header_position,
            )

            if entry.issym():
                symLinkPath = _libarchive.archive_entry_symlink(e)
                entry.symlink = symLinkPath

        finally:
            _libarchive.archive_entry_free(e)
        return entry

    @classmethod
    def from_file(cls, f, entry=None, encoding=ENCODING):
        '''Instantiates an Entry class and sets all the properties from a file on the file system.
        f can be a file-like object or a path.'''
        if entry is None:
            entry = cls(encoding=encoding)
        if entry.pathname is None:
            if isinstance(f, str):
                st = os.stat(f)
                entry.pathname = f
                entry.size = st.st_size
                entry.mtime = st.st_mtime
                entry.mode = st.st_mode
            elif hasattr(f, 'fileno'):
                st = os.fstat(f.fileno())
                entry.pathname = getattr(f, 'name', None)
                entry.size = st.st_size
                entry.mtime = st.st_mtime
                entry.mode = st.st_mode
            else:
                entry.pathname = getattr(f, 'pathname', None)
                entry.size = getattr(f, 'size', 0)
                entry.mtime = getattr(f, 'mtime', time.time())
                entry.mode = stat.S_IFREG

 
        return entry

    def to_archive(self, archive):
        '''Creates an archive header and writes it to the given archive.'''
        e = _libarchive.archive_entry_new()
        try:
            if PY3:
                _libarchive.archive_entry_set_pathname(e, self.pathname)
            else:
                _libarchive.archive_entry_set_pathname(e, self.pathname.encode(self.encoding))
            _libarchive.archive_entry_set_filetype(e, stat.S_IFMT(self.mode))
            _libarchive.archive_entry_set_perm(e, stat.S_IMODE(self.mode))
            _libarchive.archive_entry_set_size(e, self.size)
            _libarchive.archive_entry_set_mtime(e, self.mtime, 0)

            if stat.S_ISLNK(self.mode):
                _libarchive.archive_entry_set_symlink(e, self.symlink)

            call_and_check(_libarchive.archive_write_header, archive._a, archive._a, e)

        finally:
            _libarchive.archive_entry_free(e)

    def isdir(self):
        return stat.S_ISDIR(self.mode)

    def isfile(self):
        return stat.S_ISREG(self.mode)

    def issym(self):
        return stat.S_ISLNK(self.mode)

    def isfifo(self):
        return stat.S_ISFIFO(self.mode)

    def ischr(self):
        return stat.S_ISCHR(self.mode)

    def isblk(self):
        return stat.S_ISBLK(self.mode)


class Archive(object):
    '''A low-level archive reader which provides forward-only iteration. Consider
    this a light-weight pythonic libarchive wrapper.'''

    def __init__(
        self,
        f,
        mode='r',
        format=None,
        filter=None,
        entry_class=Entry,
        encoding=ENCODING,
        blocksize=BLOCK_SIZE,
        password=None,
    ):
        assert mode in ('r', 'w', 'wb', 'a'), 'Mode should be "r", "w", "wb", or "a".'
        self._stream = None
        self.encoding = encoding
        self.blocksize = blocksize
        self.password = password
        if isinstance(f, str):
            self.filename = f
            f = open(f, mode)
            # Only close it if we opened it...
            self._defer_close = True
        elif hasattr(f, 'fileno'):
            self.filename = getattr(f, 'name', None)
            # Leave the fd alone, caller should manage it...
            self._defer_close = False
        else:
            raise Exception('Provided file is not path or open file.')
        self.f = f
        self.mode = mode
        # Guess the format/filter from file name (if not provided)
        if self.filename:
            if format is None:
                format = guess_format(self.filename)[0]
            if filter is None:
                filter = guess_format(self.filename)[1]
        self.format = format
        self.filter = filter
        # The class to use for entries.
        self.entry_class = entry_class
        # Select filter/format functions.
        if self.mode == 'r':
            self.format_func = get_func(self.format, FORMATS, 0)
            if self.format_func is None:
                raise Exception('Unsupported format %s' % format)
            self.filter_func = get_func(self.filter, FILTERS, 0)
            if self.filter_func is None:
                raise Exception('Unsupported filter %s' % filter)
        else:
            # TODO: how to support appending?
            if self.format is None:
                raise Exception('You must specify a format for writing.')
            self.format_func = get_func(self.format, FORMATS, 1)
            if self.format_func is None:
                raise Exception('Unsupported format %s' % format)
            self.filter_func = get_func(self.filter, FILTERS, 1)
            if self.filter_func is None:
                raise Exception('Unsupported filter %s' % filter)
        # Open the archive, apply filter/format functions.
        self.init()

    def __iter__(self):
        while True:
            try:
                yield self.entry_class.from_archive(self, encoding=self.encoding)
            except EOF:
                break

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.denit()

    def __del__(self):
        self.close()

    def set_initial_options(self):
        pass

    def init(self):
        if self.mode == 'r':
            self._a = _libarchive.archive_read_new()
        else:
            self._a = _libarchive.archive_write_new()
        self.format_func(self._a)
        self.filter_func(self._a)
        self.set_initial_options()
        if self.mode == 'r':
            if self.password:
                if isinstance(self.password, list):
                    for pwd in self.password:
                        self.add_passphrase(pwd)
                else:
                    self.add_passphrase(self.password)
            call_and_check(_libarchive.archive_read_open_fd, self._a, self._a, self.f.fileno(), self.blocksize)
        else:
            if self.password:
                self.set_passphrase(self.password)
            call_and_check(_libarchive.archive_write_open_fd, self._a, self._a, self.f.fileno())

    def denit(self):
        '''Closes and deallocates the archive reader/writer.'''
        if getattr(self, '_a', None) is None:
            return
        try:
            if self.mode == 'r':
                _libarchive.archive_read_close(self._a)
                _libarchive.archive_read_free(self._a)
            elif self.mode == 'w':
                _libarchive.archive_write_close(self._a)
                _libarchive.archive_write_free(self._a)
        finally:
            # We only want one try at this...
            self._a = None

    def close(self, _defer=False):
        # _defer == True is how a stream can notify Archive that the stream is
        # now closed.  Calling it directly in not recommended.
        if _defer:
            # This call came from our open stream.
            self._stream = None
            if not self._defer_close:
                # We are not yet ready to close.
                return
        if self._stream is not None:
            # We have a stream open! don't close, but remember we were asked to.
            self._defer_close = True
            return
        self.denit()
        # If there is a file attached...
        if hasattr(self, 'f'):
            # Make sure it is not already closed...
            if getattr(self.f, 'closed', False):
                return
            # Flush it if not read-only...
            if hasattr(self.f, "mode") and self.f.mode != 'r' and self.f.mode != 'rb':
                if hasattr(self.f, "flush"):
                    self.f.flush()
                if hasattr(self.f, "fileno"):
                    os.fsync(self.f.fileno())
            # and then close it, if we opened it...
            if getattr(self, '_close', None):
                self.f.close()

    @property
    def header_position(self):
        '''The position within the file.'''
        return _libarchive.archive_read_header_position(self._a)

    def iterpaths(self):
        for entry in self:
            yield entry.pathname

    def read(self, size):
        '''Read current archive entry contents into string.'''
        return _libarchive.archive_read_data_into_str(self._a, size)

    def readpath(self, f):
        '''Write current archive entry contents to file. f can be a file-like object or
        a path.'''
        if isinstance(f, str):
            basedir = os.path.dirname(f)
            if not os.path.exists(basedir):
                os.makedirs(basedir)
            f = open(f, 'w')
        return _libarchive.archive_read_data_into_fd(self._a, f.fileno())

    def readstream(self, size):
        '''Returns a file-like object for reading current archive entry contents.'''
        self._stream = EntryReadStream(self, size)
        return self._stream

    def write(self, member, data=None):
        '''Writes a string buffer to the archive as the given entry.'''
        if isinstance(member, str):
            member = self.entry_class(pathname=member, encoding=self.encoding,
                mtime=time.time(), mode=stat.S_IFREG)
        if data:
            member.size = len(data)
        member.to_archive(self)

        if data:
            if PY3:
                if isinstance(data, bytes):
                    result = _libarchive.archive_write_data_from_str(self._a, data)
                else:
                    result = _libarchive.archive_write_data_from_str(self._a, data.encode(self.encoding))
            else:
                result = _libarchive.archive_write_data_from_str(self._a, data)
        _libarchive.archive_write_finish_entry(self._a)

    def writepath(self, f, pathname=None, folder=False):
        '''Writes a file to the archive. f can be a file-like object or a path. Uses
        write() to do the actual writing.'''
        member = self.entry_class.from_file(f, encoding=self.encoding)
        if isinstance(f, str):
            if os.path.isfile(f):
                f = open(f, 'r')
        if pathname:
            member.pathname = pathname
        if folder and not member.isdir():
            member.mode = stat.S_IFDIR

        if hasattr(f, 'read'):
            # TODO: optimize this to write directly from f to archive.
            self.write(member, data=f.read())
        else:
            self.write(member)

    def writestream(self, pathname, size=None):
        '''Returns a file-like object for writing a new entry.'''
        self._stream = EntryWriteStream(self, pathname, size)
        return self._stream

    def printlist(self, s=sys.stdout):
        for entry in self:
            s.write(entry.size)
            s.write('\t')
            s.write(entry.mtime.strftime(MTIME_FORMAT))
            s.write('\t')
            s.write(entry.pathname)
        s.flush()

    def add_passphrase(self, password):
        '''Adds a password to the archive.'''
        _libarchive.archive_read_add_passphrase(self._a, password)

    def set_passphrase(self, password):
        '''Sets a password for the archive.'''
        _libarchive.archive_write_set_passphrase(self._a, password)


class SeekableArchive(Archive):
    '''A class that provides random-access to archive entries. It does this by using one
    or many Archive instances to seek to the correct location. The best performance will
    occur when reading archive entries in the order in which they appear in the archive.
    Reading out of order will cause the archive to be closed and opened each time a
    reverse seek is needed.'''

    def __init__(self, f, **kwargs):
        self._stream = None
        # Convert file to open file. We need this to reopen the archive.
        mode = kwargs.setdefault('mode', 'r')
        if isinstance(f, str):
            f = open(f, mode)
        super(SeekableArchive, self).__init__(f, **kwargs)
        self.entries = []
        self.eof = False

    def __iter__(self):
        for entry in self.entries:
            yield entry
        if not self.eof:
            try:
                for entry in super(SeekableArchive, self).__iter__():
                    self.entries.append(entry)
                    yield entry
            except StopIteration:
                self.eof = True

    def reopen(self):
        '''Seeks the underlying fd to 0 position, then opens the archive. If the archive
        is already open, this will effectively re-open it (rewind to the beginning).'''
        self.denit()
        self.f.seek(0)
        self.init()

    def getentry(self, pathname):
        '''Take a name or entry object and returns an entry object.'''
        for entry in self:
            if entry.pathname == pathname:
                return entry
        raise KeyError(pathname)

    def seek(self, entry):
        '''Seeks the archive to the requested entry. Will reopen if necessary.'''
        move = entry.header_position - self.header_position
        if move != 0:
            if move < 0:
                # can't move back, re-open archive:
                self.reopen()
            # move to proper position in stream
            for curr in super(SeekableArchive, self).__iter__():
                if curr.header_position == entry.header_position:
                    break

    def read(self, member):
        '''Return the requested archive entry contents as a string.'''
        entry = self.getentry(member)
        self.seek(entry)
        return super(SeekableArchive, self).read(entry.size)

    def readpath(self, member, f):
        entry = self.getentry(member)
        self.seek(entry)
        return super(SeekableArchive, self).readpath(f)

    def readstream(self, member):
        '''Returns a file-like object for reading requested archive entry contents.'''
        entry = self.getentry(member)
        self.seek(entry)
        self._stream = EntryReadStream(self, entry.size)
        return self._stream



#!/usr/bin/env python
#
# Copyright (c) 2015, SmartFile <tcunningham@smartfile.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the organization nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from os import environ

try:
    from setuptools import setup, Extension
    from setuptools.command.build_ext import build_ext
except ImportError:
    from distutils.core import setup, Extension
    from distutils.command.build_ext import build_ext



# Use a provided libarchive else default to hard-coded path.
libarchivePrefix = environ.get('LIBARCHIVE_PREFIX')

if libarchivePrefix:
    includePath = libarchivePrefix + '/include'
else:
    includePath = '/usr/local/include'




name = 'python-libarchive'
version = '4.1.3'
readme = 'README.rst'
repourl = 'https://github.com/smartfile/python-libarchive'
long_description = open(readme).read()


class build_ext_extra(build_ext, object):
    """
    Extend build_ext allowing extra_compile_args and extra_link_args to be set
    on the command-line.
    """

    user_options = build_ext.user_options
    user_options.append(('extra-compile-args=', None, 'Extra arguments passed directly to the compiler'))
    user_options.append(('extra-link-args=', None, 'Extra arguments passed directly to the linker'))

    def initialize_options(self):
        build_ext.initialize_options(self)
        self.extra_compile_args = None
        self.extra_link_args = None

    def build_extension(self, ext):
        if self.extra_compile_args:
            ext.extra_compile_args.append(self.extra_compile_args)
        if self.extra_link_args:
            ext.extra_link_args.append(self.extra_link_args)
        super(build_ext_extra, self).build_extension(ext)


if libarchivePrefix:
    extra_compile_args = ['-I{0}/include'.format(libarchivePrefix)]
    extra_link_args = ['-Wl,-rpath={0}/lib'.format(libarchivePrefix)]
    environ['LDFLAGS'] = '-L{0}/lib {1}'.format(libarchivePrefix, environ.get('LDFLAGS', ''))
else:
    extra_compile_args = []
    extra_link_args = ['-l:libarchive.so']

__libarchive = Extension(
    name='libarchive.__libarchive',
    sources=['libarchive/_libarchive_wrap.c'],
    libraries=['archive'],
    extra_compile_args=extra_compile_args,
    extra_link_args=extra_link_args,
    include_dirs=[includePath, 'libarchive'],
)


setup(
    name=name,
    version=version,
    description='A libarchive wrapper for Python supporting password protection.',
    long_description=long_description,
    license='BSD-style license',
    platforms=['any'],
    author='Vadim Lebedev, Ben Timby, Travis Cunningham, Ryan Johnston, SmartFile',
    author_email='tech@smartfile.com',
    url=repourl,
    packages=['libarchive'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: C',
        'Programming Language :: Python',
        'Topic :: System :: Archiving :: Compression',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    cmdclass={
        'build_ext': build_ext_extra,
    },
    ext_modules=[__libarchive],
)



import os, time
from libarchive import is_archive, Entry, SeekableArchive, _libarchive
from zipfile import ZIP_STORED, ZIP_DEFLATED


def is_zipfile(filename):
    return is_archive(filename, formats=('zip',))


class ZipEntry(Entry):
    def __init__(self, *args, **kwargs):
        super(ZipEntry, self).__init__(*args, **kwargs)

    def get_filename(self):
        return self.pathname

    def set_filename(self, value):
        self.pathname = value

    filename = property(get_filename, set_filename)

    def get_file_size(self):
        return self.size

    def set_file_size(self, value):
        assert isinstance(value, int), 'Please provide size as int or long.'
        self.size = value

    file_size = property(get_file_size, set_file_size)

    def get_date_time(self):
        return time.localtime(self.mtime)[0:6]

    def set_date_time(self, value):
        assert isinstance(value, tuple), 'mtime should be tuple (year, month, day, hour, minute, second).'
        assert len(value) == 6, 'mtime should be tuple (year, month, day, hour, minute, second).'
        self.mtime = time.mktime(value + (0, 0, 0))

    date_time = property(get_date_time, set_date_time)

    header_offset = Entry.header_position

    def _get_missing(self):
        raise NotImplemented()

    def _set_missing(self, value):
        raise NotImplemented()

    compress_type = property(_get_missing, _set_missing)
    comment = property(_get_missing, _set_missing)
    extra = property(_get_missing, _set_missing)
    create_system = property(_get_missing, _set_missing)
    create_version = property(_get_missing, _set_missing)
    extract_version = property(_get_missing, _set_missing)
    reserved = property(_get_missing, _set_missing)
    flag_bits = property(_get_missing, _set_missing)
    volume = property(_get_missing, _set_missing)
    internal_attr = property(_get_missing, _set_missing)
    external_attr = property(_get_missing, _set_missing)
    CRC = property(_get_missing, _set_missing)
    compress_size = property(_get_missing, _set_missing)

# encryption is one of (traditional = zipcrypt, aes128, aes256)
class ZipFile(SeekableArchive):
    def __init__(self, f, mode='r', compression=ZIP_DEFLATED, allowZip64=False, password=None,
        encryption=None):
        self.compression = compression
        self.encryption = encryption
        super(ZipFile, self).__init__(
            f, mode=mode, format='zip', entry_class=ZipEntry, encoding='CP437', password=password
        )
        

    getinfo = SeekableArchive.getentry

    def set_initial_options(self):
        if self.mode == 'w' and self.compression == ZIP_STORED:
            # Disable compression for writing.
            _libarchive.archive_write_set_format_option(self._a, "zip", "compression", "store")
        
        if self.mode == 'w' and self.password:
            if not self.encryption:
                self.encryption = "traditional"
            _libarchive.archive_write_set_format_option(self._a, "zip", "encryption", self.encryption)
          
       

    def namelist(self):
        return list(self.iterpaths())

    def infolist(self):
        return list(self)

    def open(self, name, mode, pwd=None):
        if mode == 'r':
            if pwd:
                self.add_passphrase(pwd)
            return self.readstream(name)
        else:
            return self.writestream(name)

    def extract(self, name, path=None, pwd=None):
        if pwd:
            self.add_passphrase(pwd)
        if not path:
            path = os.getcwd()
        return self.readpath(name, os.path.join(path, name))

    def extractall(self, path, names=None, pwd=None):
        if pwd:
            self.add_passphrase(pwd)
        if not names:
            names = self.namelist()
        if names:
            for name in names:
                self.extract(name, path)

    def read(self, name, pwd=None):
        if pwd:
            self.add_passphrase(pwd)
        return super(ZipFile, self).read(name)

    def writestr(self, member, data, compress_type=None):
        if compress_type != self.compression and not (compress_type is None):
            raise Exception('Cannot change compression type for individual entries.')
        return self.write(member, data)

    def setpassword(self, pwd):
        return self.set_passphrase(pwd)

    def testzip(self):
        raise NotImplemented()

    def _get_missing(self):
        raise NotImplemented()

    def _set_missing(self, value):
        raise NotImplemented()

    comment = property(_get_missing, _set_missing)
